/* 
 * Program:             $RCSfile: AR3Attr.h,v $  $Revision: 30.11 $
 * 
 * Purpose:             
 * 
 * Author:              S.Yamamoto  1991/09/30
 * 
 * (C) Copyright:       S.Yamamoto  1991 - 2000
 *                      This file is a product of the project Sapid.
 */

#if !defined(_AR3ATTR_H) && defined(SAPID_AR3)
#define _AR3ATTR_H

#define BLOCK_BASIC         (1 << 0)
#define BLOCK_IF            (1 << 1)
#define BLOCK_SWITCH        (1 << 2)
#define BLOCK_FOR           (1 << 3)
#define BLOCK_WHILE         (1 << 4)
#define BLOCK_DO            (1 << 5)
#define BLOCK_DECL          (1 << 6)
#define BLOCK_COMP          (1 << 7)

#define BLOCK_BRANCH        (BLOCK_IF | BLOCK_SWITCH | BLOCK_FOR | \
                             BLOCK_WHILE | BLOCK_DO | BLOCK_DECL)
#define BLOCK_HIERACHICAL   (BLOCK_BRANCH | BLOCK_COMP)

/******************************************************************************/

#define CONST_INT           1
#define CONST_FLOAT         2
#define CONST_CHAR          3
#define CONST_STRING        4
#define CONST_ENUM          5
#define CONST_WCHAR         6

/******************************************************************************/

#define EXPR_EMPTY          1
#define EXPR_BREAK          2
#define EXPR_CONTINUE       3
#define EXPR_RETURN         4
#define EXPR_GOTO           5

#define EXPR_SORT_ARITH     6
#define EXPR_SORT_LOGICAL   7
#define EXPR_SORT_ARRAY     8
#define EXPR_SORT_MEMBER    9
#define EXPR_SORT_POINTER   10
#define EXPR_SORT_ADDRESS   11
#define EXPR_SORT_TRI       12
#define EXPR_SORT_SIZEOF    13
#define EXPR_SORT_CAST      14
#define EXPR_SORT_COMMA     15
#define EXPR_SORT_FUNCCALL  16
#define EXPR_SORT_INITLIST  17
#define EXPR_SORT_BUILTIN   18
#define EXPR_SORT_BIT       19
#define EXPR_SORT_SHIFT     20
#define EXPR_SORT_COMPARE   21
#define EXPR_SORT_POINTER_MEMBER    22

/* Added by S.Yamamoto 1996/05/24. */ 
#define EXPR_LR_UNKNOWN     0

#define EXPR_L_VALUE        (1 << 0)
#define EXPR_R_VALUE        (1 << 1)
#define EXPR_PL_VALUE       (1 << 2)
#define EXPR_PR_VALUE       (1 << 3)
#define EXPR_P_REF          (1 << 4)
#define EXPR_LR_VALUE       (EXPR_L_VALUE | EXPR_R_VALUE)
#define EXPR_PLR_VALUE      (EXPR_PL_VALUE | EXPR_PR_VALUE)

/******************************************************************************/

/* 
 * DECL_ENUM is commented by S.Yamamoto 1996/05/21.
 * 
 * Note, DECL_MACRO is used in "<SAPID_TOP>/AR3/mkmacro.c".
 * DECL_MACRO is commented by S.Yamamoto 2000/04/03.
 */

#define DECL_NORMAL         1   /* Declaration only. */
#define DECL_FUNCDEF        2   /* Declaration and Declarator. */
#define DECL_MEMBER         3   /* Declaration and Declarator. */
/* #define DECL_ENUM           4 */
#define DECL_ENUMERATOR     5   /* Declaration and Declarator. */
#define DECL_PARAMETER      6   /* Declaration only. */
#define DECL_PROTOTYPE      7   /* Declarator only. */
#define DECL_TAG            8   /* Declarator only. */
#define DECL_VARIABLE       9   /* Declarator only. */
#define DECL_TYPE           10  /* Declaration and Declarator. */
#define DECL_PARMTYPE       11  /* Declarator "*" of "int f(char *);". */
/* #define DECL_MACRO          999 */

#define OPT_STRUCT          1
#define OPT_FUNCDEF         2
#define OPT_INITIALIZER     3
#define OPT_PROTOTYPE       4
#define OPT_FUNCPTR         5

#define STORAGE_NONE        0
#define STORAGE_AUTO        1
#define STORAGE_EXTERN      2
#define STORAGE_TYPEDEF     3
#define STORAGE_STATIC      4
#define STORAGE_REGISTER    5

/******************************************************************************/

/* 
 * ID_ENUM is renamed to ID_ENUMERATOR 1996/05/21.
 */

#define ID_VARIABLE         (1 << 0)
#define ID_FUNCTION         (1 << 1)
#define ID_MEMBER           (1 << 2)
#define ID_ENUMERATOR       (1 << 3)
#define ID_TYPENAME         (1 << 4)
#define ID_TAG              (1 << 5)

/*
 * ID_SCOPE_XXX for ISDB 1998/01/24.
 */

#define ID_SCOPE_NULL_VALUE 0
#define ID_SCOPE_GLOBAL     1
#define ID_SCOPE_NOT_GLOBAL 2

/******************************************************************************/

#define LABEL_NAMED         1
#define LABEL_CASE          2
#define LABEL_DEFAULT       3

/* 
 * Operators.
 */

typedef enum {
    OP_ARROW = 1, OP_DOT,                       /* -> .              1 -  2 */
    OP_NOT, OP_B_NOT, OP_INC, OP_DEC, OP_PLUS,  /* ! ~ ++ -- +       3 -  7 */
    OP_MINUS, OP_POINTER, OP_ADDRESS,           /* - * &             8 - 10 */
    OP_MUL, OP_DIV, OP_MOD, OP_ADD, OP_SUB,     /* * / % + -        11 - 15 */
    OP_SHL, OP_SHR,                             /* << >>            16 - 17 */
    OP_LT, OP_LE, OP_GT, OP_GE, OP_EQ, OP_NE,   /* < <= > >= == !=  18 - 23 */
    OP_B_AND, OP_B_XOR, OP_B_OR,                /* & ^ |            24 - 26 */
    OP_L_AND, OP_L_OR,                          /* && ||            27 - 28 */
    OP_ASSIGN, OP_A_ADD, OP_A_SUB,              /* = += -=          29 - 31 */
    OP_A_MUL, OP_A_DIV, OP_A_MOD,               /* *= /= %=         32 - 34 */
    OP_A_AND, OP_A_XOR, OP_A_OR,                /* &= ^= |=         35 - 37 */
    OP_A_SHL, OP_A_SHR,                         /* <<= >>=          38 - 39 */
    OP_COMMA, OP_SIZEOF,                        /* , sizeof         40 - 41 */
    OP_SIZEOF_TYPE, OP_CAST,                    /* sizeof(t) (t)    42 - 43 */
    OP_L_PAR, OP_R_PAR, OP_L_ARRAY, OP_R_ARRAY, /* ( ) [ ]          44 - 47 */
    OP_QUESTION, OP_COLON,                      /* ? :              48 - 49 */
    OP_BUILTIN,                                 /* __builtin        50 */
    OP_BUILTIN_TYPE                             /* __builtin(t)     51 */
} C_OPERATOR_ENUM;

#define OP_LAST     OP_BUILTIN_TYPE

/******************************************************************************/

#define TYPE_UNKNOWN        0

#define TYPE_DECLARATION    1
#define TYPE_POINTER        2
#define TYPE_ARRAY          3
#define TYPE_FUNC           4
#define TYPE_STRUCT_TAG     5
#define TYPE_TYPE_DEF       6
#define TYPE_MEMBER         7
#define TYPE_ENUM_SPECIFIER 8
#define TYPE_UNION_TAG      9
#define TYPE_BUILTIN       10

#define TYPE_VOID           1               /* Type. */
#define TYPE_INT            2
#define TYPE_CHAR           3
#define TYPE_FLOAT          4
#define TYPE_DOUBLE         5
#define TYPE_DOTS           6
#define TYPE_BUILTIN_TYPE   7

#define TYPE_SHORT          1               /* Length. */
#define TYPE_LONG           2

#ifdef ACCEPT_TYPE_LONG_LONG
#define TYPE_LONG_LONG      3
#endif

#define TYPE_SIGNED         1               /* Signedness. */
#define TYPE_UNSIGNED       2

#define TYPE_CONST          1               /* Qualifier. */
#define TYPE_VOLATILE       2

/******************************************************************************/

/* Scope. */

#define VAR_GLOBAL          1
#define VAR_LOCAL           2
/* #define FUNC_RET            3 */

/* Storage class. */

#define VAR_AUTO            1
#define VAR_REGISTER        2
#define VAR_STATIC          3
#define VAR_EXTERN          4

#endif /* !defined(_AR3ATTR_H) && defined(SAPID_AR3) */
