/* 
 * Program:             $RCSfile: SpdUtil.h,v $  $Revision: 30.100 $
 * 
 * Purpose:             Sapid utility functions.
 * 
 * Author:              T.Mimura  1993/12/01
 *                      S.Yamamoto  1994/09/01
 * Modified             T.Hidaka  1998/05/04
 * 
 * (C) Copyright:       T.Mimura, T.Hidaka and S.Yamamoto  1993 - 2003
 *                      This file is a product of the project Sapid.
 */

#if !defined(_SPD_UTIL_H)
#define _SPD_UTIL_H

#include <sys/types.h>      /* For stat(). */
#include <sys/stat.h>       /* For stat(). */

/* Debug flag for memory management.  */
/* #define SPD_DEBUG_ALLOC */

/******************************************************************************/

/* In "simplifyPath.c". */

SpdString spdSimplifyPath(SpdConstString path);

/* In "path.c". */

char        *spdStrcatDup(const char *str1, const char *str2);
char        *spdCatPath(const char *dir, const char *fname);
char        *spdRemoveDriveName(char *orgPath);

SpdBoolean  spdIsDirectory(const char *dname);
SpdBoolean  spdIsFile(const char *path);
void        spdMkdir(SpdConstString path);
SpdBoolean  spdMkdirR(SpdConstString path);
void        spdUnlink(SpdConstString path);
void        spdSymlink(SpdConstString name1, SpdConstString name2);

void        spdStat(const char *path, struct stat *buf);
void        spdFstat(int filedes, struct stat *buf);

#if !defined(WIN32)
void        spdLstat(const char *path, struct stat *buf);
#endif

/* Added by S.Yamamoto 1999/05/24. */
SpdConstString  spdBasename(SpdConstString path);

/* Added by S.Yamamoto 2001/07/26. */
SpdBoolean  spdIsFileExtension(SpdConstString file, SpdConstString ext);
SpdConstString  spdCnvFileExtension(SpdConstString file,
                                    SpdConstString oldExt,
                                    SpdConstString newExt);
SpdString   spdReplaceFileExtension(SpdString file,
                                    SpdConstString oldExt,
                                    SpdConstString newExt);

/*
 * General initialization.
 */

void spdInitialize(const char *this_prog);
/* void spdCleanUp(const char *target_prog); */

/*
 * Memory management functions.
 */

void        *spdMallocSub(size_t size);
void        *spdReallocSub(void *ptr, size_t size);
void        *spdCallocSub(size_t nelem, size_t elsize);
void        *spdSafetyFreeSub(const void *ptr);
void        spdIncRegionIntSub(void **ptr, int *nelem, int inc, size_t elsize);

void        *spdMallocDbg(char *file, int line, size_t size);
void        *spdReallocDbg(char *file, int line, void *ptr, size_t size);
void        *spdCallocDbg(char *file, int line, size_t nelem, size_t elsize);
void        spdFreeDbg(char *file, int line, void *ptr);
void        *spdSafetyFreeDbg(char *file, int line, void *ptr);
void        spdIncRegionIntDbg(char *file, int line, void **ptr,
                               int *nelem, int inc, size_t elsize);

#if defined(SPD_DEBUG_ALLOC)

#define spdMalloc(size) \
        spdMallocDbg(__FILE__, __LINE__, (size))

#define spdRealloc(ptr, size) \
        spdReallocDbg(__FILE__, __LINE__, (ptr), (size))

#define spdCalloc(nelem, elsize) \
        spdCallocDbg(__FILE__, __LINE__, (nelem), (elsize))

#define spdFree(ptr) \
        spdFreeDbg(__FILE__, __LINE__, (void *)(ptr))

#define spdSafetyFree(ptr) \
        spdSafetyFreeDbg(__FILE__, __LINE__, (ptr))

#define spdIncRegionInt(ptr, nelem, inc, elsize) \
        spdIncRegionIntDbg(__FILE__, __LINE__, (ptr), (nelem), (inc), (elsize))

#else /* !defined(SPD_DEBUG_ALLOC) */

#define spdMalloc(size)                 spdMallocSub((size_t)(size))
#define spdRealloc(ptr, size)           spdReallocSub((ptr), (size_t)(size))
#define spdCalloc(nelem, elsize)        spdCallocSub((size_t)(nelem), (size_t)(elsize))
#define spdFree(ptr)                    free((void *)(ptr))
#define spdSafetyFree(ptr)              spdSafetyFreeSub((ptr))
#define spdIncRegionInt(ptr, nelem, inc, elsize) \
        spdIncRegionIntSub((ptr), (nelem), (inc), (size_t)(elsize))

#endif

/*
 * String manipulation functions.
 */

char        *spdStrdup(const char *str);
char        *spdSafetyStrdup(const char *str);
char        *spdStrcpyUntilChr(char *to, const char *from, int c);
char        *spdStrdupUntilChr(const char *str, int c);
char        *spdStrcat(char *a, char *b);
int         spdSafetyStrcmp(const char *a, const char *b);
SpdBoolean  spdSafetyStreq(const char *a, const char *b);
SpdConstString spdItoa(int x);
SpdConstString spdObjIdStr(SpdObjId id);

/*
 * Messaging and error trap functions.
 */

void        spdMessage(const char *format, ...);
void        spdWarning(const char *format, ...);
void        spdBugExit(const char *format, ...);
void        spdError(const char *format, ...);

void        spdErrorExit(const char *format, ...);
void        _spdErrorExit(const char *file, int line, const char *format, ...);

#if defined(__GNUC__) && defined(USE_VARARG_MACRO)
#define spdErrorExit(fmt...)       _spdErrorExit(__FILE__, __LINE__, ## fmt)
#endif 

void        spdMessageCont(const char *format, ...);

/* In "spdInfo.c". */

struct _SpdInfoData;

typedef struct {
    char                filename[FILENAME_MAX + 1];
    SpdBoolean          modified;
    struct _SpdInfoData *data;
} SpdInfo;

#ifdef DAPID

#define MAX_ATTRNAME_LEN    128

typedef struct _SpdInfoCursor {
    SpdInfo     *info;
    int         start_num;
    char        attrname[MAX_ATTRNAME_LEN + 1];
} SpdInfoCursor;

void spdAddInfoInt(SpdInfo *info, SpdString attrName, int attrValue);
void spdAddInfoString(SpdInfo *info, SpdString attrName, SpdConstString attrValue);
SpdInfoCursor spdGetInfoStringFirst(SpdInfo *info, SpdString attrname);
SpdString spdGetInfoStringNext(SpdInfoCursor *cur);

#endif /* DAPID */

SpdString spdGetInfoString(SpdInfo *info, SpdConstString attrName);
int spdGetInfoInt(SpdInfo *info, SpdConstString attrName, int defaultValue);
void spdSetInfoString(SpdInfo *info,
                      SpdConstString attrName, SpdConstString attrValue);
void spdSetInfoInt(SpdInfo *info, SpdConstString attrName, int attrValue);
void spdCloseInfo(SpdInfo *info);

int spdGetInfoNumOfObjs(SpdInfo *info, SpdConstString className);
void spdSetInfoNumOfObjs(SpdInfo *info, SpdConstString className, int nums);

/* In "spdInfo.c". */

void spdSetSDBinfoFile(SpdConstString dir, SpdConstString baseName);
void spdResetSDBinfoFile(void);
SpdInfo *spdOpenSDBinfo(void);
SpdInfo *spdOpenISDBinfo(SpdObjId fileId);
SpdInfo *spdOpenPIDBinfo(SpdObjId fileId);

FILE *spdOpenDSDB(char *dname, int mode);
SpdInfo *spdOpenDSDBinfo(SpdConstString fileName);
SpdInfo *spdOpenDSDBsrcfraginfo(SpdConstString fileName);

SpdInfo *spdOpenJSDBinfo(const char *frejaname);

/* spdOption(). */

#include <Sapid/spdOption.h>

/* Memory profilie. */

void        spdInitMemProf(void);
void        _spdSafetyInitMemProf(void);
void        spdPrintMemProf(void);

/* SpdBoolean  _spdIsInitializedMemProf(void); */

/* SDB directory. */

void        spdSetSDBdir(SpdConstString dir);
SpdString   spdGetSDBdir(void);

/* spdSDBformat.c */

void spdSetReadSDBformat(int format);
int spdGetReadSDBformat(void);
void spdSetWriteSDBformat(int format);
int spdGetWriteSDBformat(void);

/* spdGetClassMacroName.c */

SpdConstString spdGetClassMacroName(SpdClassId classId);
SpdConstString spdGetModelMacroName(SpdModelId modelId);

/* spdOpenSDBfile.c */

/* #define SPD_MY_FSEEK */
/* #define DEBUG_MY_FSEEK */

#ifdef SPD_MY_FSEEK

#if defined(lint)
#define DW_MY_FSEEK         printf      /* Just for cheking. */
#elif defined(DEBUG_MY_FSEEK)
#define DW_MY_FSEEK         spdMessage
#else
#define DW_MY_FSEEK         (void)
#endif

typedef struct {
    SpdBoolean      isOpened;
    SpdObjId        fileId;
} SpdOpendFile;

extern SpdOpendFile     spdOpendFile[FOPEN_MAX];

#endif /* SPD_MY_FSEEK */

SpdBoolean spdCheckSDBClassFile(const char *prefix, const char *className);

FILE *spdOpenSDBfile(const char *prefix, const char *filename, int mode);
FILE *spdOpenISDB(SpdObjId ifileId, int mode);
FILE *spdOpenSourceById(SpdObjId fileId, int mode);
FILE *spdOpenJavaSource(SpdConstString fileName, int mode);
FILE *spdOpenFreja(const char *prefix, const char *className, int mode);

FILE *spdFopen(const char *fileName, char *type);
FILE *spdSimpleFopen(const char *fileName, char *type);
int spdFclose(FILE *fp);

int lineColumnToChar(char *text, int length, int line, int column);
int charToLineColumn(char *text, int length, int c,
             int *line, int *column);
/* Function name getBeginningOfLine() or getBegOfLine() is better. */
char *getHeadOfLine(char *text, int length, int line);

/******************************************************************************/

/* spdIO.c */

#ifdef SAPID_AR3_KERNEL

int spdReadIntAttr(FILE *fp);
int spdReadIntAttrText(FILE *fp);
int spdReadIntAttrBinary(FILE *fp);

long spdReadLongAttr(FILE *fp);
long spdReadLongAttrText(FILE *fp);
long spdReadLongAttrBinary(FILE *fp);

const char *spdReadStrAttr(FILE *fp);
char *spdReadStrAttrText(FILE *fp);
char *spdReadStrAttrBinary(FILE *fp);

void spdWriteIntAttr(FILE *fp, int value);
void spdWriteIntAttrText(FILE *fp, int value);
void spdWriteIntAttrBinary(FILE *fp, int value);

void spdWriteLongAttr(FILE *fp, long value);
void spdWriteLongAttrText(FILE *fp, long value);
void spdWriteLongAttrBinary(FILE *fp, long value);

void spdWriteStrAttr(FILE *fp, const char *value);
void spdWriteStrAttrText(FILE *fp, const char *value);
void spdWriteStrAttrBinary(FILE *fp, const char *value);

#endif /* SAPID_AR3_KERNEL */

/******************************************************************************/

#ifdef DAPID
char *makeDocIfilename(const char *docfilename, const int offset);
int getSrcElemObjId(SpdConstString ifilename);
#endif

/* Basic types. */

#include <Sapid/spdQueue.h>
#include <Sapid/spdSet.h>
#include <Sapid/spdSet3.h>
#include <Sapid/spdStringPool.h>

#if defined(USE_SPDSET1)

typedef SpdSet1         SpdSet;

#define spdSetContains(s, k)            spdSetContains1((s), (k))
#define spdAddElem(s, k, c)             spdAddElem1((s), (k), (c))
#define spdDelElem(s, k)                spdDelElem1((s), (k))
#define spdCreateSet(size)              spdCreateSet1((size))
#define spdFreeSet(s)                   spdFreeSet1((s))
#define spdSearchElem(s, k)             spdSearchElem1((s), (k))
#define spdGetArrayOfElem(s)            spdGetArrayOfElem1((s))
#define spdGetAElem(s, ak)              spdGetAElem1((s), (ak))

#define spdDupSet(s)                    spdDupSet1((s))
#define spdUnionSet(s1, s2)             spdUnionSet1((s1), (s2))
#define spdIntersectionSet(s1, s2)      spdIntersectionSet1((s1), (s2))
#define spdDifferenceSet(s1, s2)        spdDifferenceSet1((s1), (s2))

#define spdClearSetEnumeration(s)       spdClearSetEnumeration1((s));

#define SAPID_EMPTY_SET                 SAPID_EMPTY_SET1

#elif defined(USE_SPDSET3)

typedef SpdSet3         SpdSet;

#define spdSetContains(s, k)            spdSetContains3((s), (k))
#define spdAddElem(s, k, c)             spdAddElem3((s), (k), (c))
#define spdDelElem(s, k)                spdDelElem3((s), (k))
#define spdCreateSet(size)              spdCreateSet3((size))
#define spdFreeSet(s)                   spdFreeSet3((s))
#define spdSearchElem(s, k)             spdSearchElem3((s), (k))
#define spdGetArrayOfElem(s)            spdGetArrayOfElem3((s))
#define spdGetAElem(s, ak)              spdGetAElem3((s), (ak))

#define spdDupSet(s)                    spdDupSet3((s))
#define spdUnionSet(s1, s2)             spdUnionSet3((s1), (s2))
#define spdIntersectionSet(s1, s2)      spdIntersectionSet3((s1), (s2))
#define spdDifferenceSet(s1, s2)        spdDifferenceSet3((s1), (s2))

#define spdClearSetEnumeration(s)       spdClearSetEnumeration3((s));

#define SAPID_EMPTY_SET                 SAPID_EMPTY_SET3

#endif /* defined(USE_SPDSET3) */

/* Misc. functions. */

#include <Sapid/spdFseek.h>

/* Added by S.Yamamoto 1999/01/11. */
/* FreeBSD-libkern-qsort.c */

void spdQsort(void *a, size_t n, size_t es,
              int (*cmp)(const void *, const void *));

/*
 * For gettimeofday().
 * Added by S.Yamamoto 1999/08/06.
 */

#if defined(HAVE_SYS_TIME_H)
#include <sys/time.h>
#endif

/*
 * For times().
 * Added by S.Yamamoto 1999/08/06.
 * 
 * Note, times() will be superseded by getrusage().
 */

#if defined(HAVE_SYS_TIMES_H)
#include <sys/types.h>      /* <sys/types.h> must stay before <sys/times.h>. */
#include <sys/times.h>
#endif

/*
 * For supporting Japanese code.
 */

#include <Sapid/spdEncode.h>

SpdEncode spdDetectEncoding(FILE *in);

/******************************************************************************/

#if defined(__hpux)

#include <netinet/in.h>

#elif defined(__sgi)

#include <sys/types.h>
#include <netinet/in.h>

#elif defined(__sun)

#include <sys/types.h>
#include <netinet/in.h>

#elif defined(__FreeBSD__)

#include <sys/param.h>

#elif defined(__linux__)

#include <netinet/in.h>

#elif defined(WIN32) && !defined(__CYGWIN__) 

#define DECLSPEC_IMPORT         __declspec(dllimport)
#define WINSOCK_API_LINKAGE     DECLSPEC_IMPORT
#define far
#define FAR                     far
#define PASCAL                  __stdcall
#define WSAAPI                  FAR PASCAL

typedef unsigned long   u_long;

WINSOCK_API_LINKAGE u_long WSAAPI htonl(u_long hostlong);
WINSOCK_API_LINKAGE u_long WSAAPI ntohl(u_long netlong);

#else /* Unknown UNIX architectures. */

#include <netinet/in.h>

#endif /* Unknown architecture. */

#define SPD_HTON_INT(x)         ((int)htonl((unsigned long)(x)))
#define SPD_NTOH_INT(x)         ((int)ntohl((unsigned long)(x)))

#define SPD_HTON_LONG(x)        (htonl((x)))
#define SPD_NTOH_LONG(x)        (ntohl((x)))

#endif /* !defined(_SPD_UTIL_H) */
