/* 
 * Program:             $RCSfile: sdaarea.h,v $  $Revision: 31.6 $
 * 
 * Purpose:
 *
 * Author:              T.Suzuki  1998/10/22
 *                      S.Yamamoto  1997/10/22
 *
 * (C) Copyright:       T.Suzuki and S.Yamamoto  1998 - 2003
 *                      This file is a product of the project Sapid.
 */

#ifndef _SPD_SAREA_H
#define _SPD_SAREA_H

/*
 * SDA area model:
 *   ______        ___ <expr_sarea>----[expr]
 *  +------[sArea]----<ident_sarea>----[ident]
 *  ||        |
 *  ||       / \
 *  ||     +---------+--------+---------+------+
 *  ||     |         |        |         |      |
 *  ||  [NORMAL]  [STRUCT] [POINTER] [ARRAY] [FUNC]
 *  ||               |        |         |
 *  ||               |        |         |
 *  |+--<consist_of>-+--------|---------+
 *  |+----<sarea_sarea>-------+
 *  +-----<alias_to>----------+
 *
 *  Examples:
 *    int i;
 *    int *j;
 *    struct S {int p; char *s} t, *u;
 *    struct V {struct *V v;} x;
 *    [ident i] -- [area i]
 *    [ident j] -- [area j] -- [area *j]
 *    [ident t] -- [area t] -- [area p]
 *                         \-- [area s] -- [area *s]
 *    [ident u] -- [area u] -- [area *u]
 *    [ident x] -- [area x] -- [area v] -- [area *v] :: (recurcive stop)
 *
 *    j = &i;
 *    j = &t.p;
 *    [area *j]--<alias_to>--[area i]
 *    [area *j]--<alias_to>--[area p of t]
 */

/*
 * ToDo:
 *     self recursive.
 */

/* Class id. */

#define SPD_CID_SAREA               (SPD_FIRST_CID_SA_MODEL + 0)

#if (SPD_CID_SAREA != SPD_LAST_CID_SA_MODEL)
    "Invalid configuration."
#endif

#define SPD_CID_NUM_SA_MODEL \
        (SPD_LAST_CID_SA_MODEL - SPD_FIRST_CID_SA_MODEL + 1)

/* Relation id. */

#define SPD_RID_IDENT_SAREA         (SPD_FIRST_RID_SA_MODEL + 0)
#define SPD_RID_CONSIST_OF          (SPD_FIRST_RID_SA_MODEL + 1)
#define SPD_RID_SAREA_SAREA         (SPD_FIRST_RID_SA_MODEL + 2)
/* 
 * #define SPD_RID_ALIAS_TO            (SPD_FIRST_RID_SA_MODEL + 3)
 * #define SPD_RID_EXPR_SAREA          (SPD_FIRST_RID_SA_MODEL + 4)
 */

#if (SPD_RID_SAREA_SAREA != SPD_LAST_RID_SA_MODEL)
    "Invalid configuration."
#endif

#define SPD_RID_NUM_SA_MODEL \
        (SPD_LAST_RID_SA_MODEL - SPD_FIRST_RID_SA_MODEL + 1)

/* Number of all classes and relations. */

#define SPD_CRID_NUM_SA_MODEL \
        (SPD_CID_NUM_SA_MODEL + SPD_RID_NUM_SA_MODEL)

/******************************************************************************/

/* Attribute value of classs "sarea". */

#define SAREA_NORMAL    0
#define SAREA_STRUCT    1
#define SAREA_POINTER   2
#define SAREA_ARRAY     3
#define SAREA_FUNC      4

/******************************************************************************/

#define SPD_SDA_AREA_MODEL_VERSION  1

extern SpdDBModel SDAarea_model;

#endif /* _SPD_SAREA_H */
