/* 
 * Program:             $RCSfile: spdQueue.h,v $  $Revision: 30.13 $
 * 
 * Purpose:             
 * 
 * Author:              T.Suzuki  1997/06/25
 *                      S.Yamamoto  1997/06/27
 * 
 * (C) Copyright:       T.Suzuki and S.Yamamoto  1997 - 2000
 *                      This file is a product of the project Sapid.
 */

#ifndef _SPD_QUEUE_H
#define _SPD_QUEUE_H

#define SPD_EMPTY_QUEUE     {0, 0, 0, 0, NULL}

#define QUEUE_ALLOC_SIZE    8192

#define SPD_QUEUE_UNION_TYPE_ELEM

#ifdef SPD_QUEUE_UNION_TYPE_ELEM

typedef union {
    int   i;
    void *p;
} SpdQueueElemType;

#define SPD_QUEUE_NIL_VALUE   {SPD_NON_ID}

#else /* not SPD_QUEUE_UNION_TYPE_ELEM */

typedef int SpdQueueElemType;

#define SPD_QUEUE_NIL_VALUE   SPD_NON_ID

#endif /* SPD_QUEUE_UNION_TYPE_ELEM */

typedef struct {
  int   head;
  int   tail;
  int   elem;
  int   max;
  SpdQueueElemType *buff;
} SpdQueue;

/*    Low <==   ==> High       : Address
                     []-->head : push_back  (enqueue)
           []-->tail           : pop_front  (dequeue)
     [tail]<--                 : push_front
               [head]<--       : pop_back
  ---+----+----+----+----+---
     |    |tail|    |head|
     |    |NN  |NN  |    |
  ---+----+----+----+----+---
 */

/******************************************************************************/

extern SpdQueueElemType     _spd_queue_dummy;

/******************************************************************************/

SpdQueue spdQueueInit(void);

void     _spdQueuePushBack(SpdQueue *queue, SpdQueueElemType val);
void     _spdQueuePushFront(SpdQueue *queue, SpdQueueElemType val);
SpdQueueElemType _spdQueuePopBack(SpdQueue *queue);
SpdQueueElemType _spdQueuePopFront(SpdQueue *queue);
void     spdQueueClean(SpdQueue *queue);

#define spdQueueEnqueue(q, val) spdQueuePushFront((q), (val))
#define spdQueueDequeue(q)      spdQueuePopBack(q)

/* For debug. */
void spdQueue_print_info(void);

/* Now checking. */
void spdQueueSort(SpdQueue *queue, int (*compar)(const SpdQueueElemType *,
                                                 const SpdQueueElemType *));
int  _spdQueueSearchIdx(SpdQueue *queue, SpdQueueElemType val);
SpdQueueElemType _spdQueuePeek(SpdQueue *queue, int pos);
SpdQueueElemType _spdQueuePoke(SpdQueue *queue, int pos, SpdQueueElemType val);
SpdQueueElemType* spdQueueToArray(SpdQueue *queue);

SpdQueueElemType _spdQueueGetFront(SpdQueue *queue);
SpdQueueElemType _spdQueueGetBack(SpdQueue *queue);
void             spdQueueDelete(SpdQueue *queue, int s_pos, int len);

/* Not implemented yet. */
void spdQueueSplice(SpdQueue *queue, int pos, int len, int* dt, int dtlen);

/* Wrapper for union. */
#define SPD_QUEUE_USE_UNION

#ifdef SPD_QUEUE_UNION_TYPE_ELEM

#define spdQueuePushBack(q, v) \
        _spdQueuePushBack((q), (_spd_queue_dummy.i = (v), _spd_queue_dummy))
#define spdQueuePushFront(q, v) \
        _spdQueuePushFront((q), (_spd_queue_dummy.i = (v), _spd_queue_dummy))
#define spdQueuePopBack(q)      (_spdQueuePopBack((q)).i)
#define spdQueuePopFront(q)     (_spdQueuePopFront((q)).i)
#define spdQueuePeek(q, p)      (_spdQueuePeek((q), (p)).i)
#define spdQueuePoke(q, p, v) \
        (_spdQueuePoke((q), (p), \
                       (_spd_queue_dummy.i = (v), _spd_queue_dummy)).i)
#define spdQueueGetFront(q)     (_spdQueueGetFront((q)).i)
#define spdQueueGetBack(q)      (_spdQueueGetBack((q)).i)
#define spdQueueSearchIdx(q, v) \
        _spdQueueSearchIdx((q), (_spd_queue_dummy.i = (v), _spd_queue_dummy))

#else /* not SPD_QUEUE_UNION_TYPE_ELEM */

#define spdQueuePushBack(q, v)  _spdQueuePushBack((q), (v))
#define spdQueuePushFront(q, v) _spdQueuePushFront((q), (v))
#define spdQueuePopBack(q)      _spdQueuePopBack((q))
#define spdQueuePopFront(q)     _spdQueuePopFront((q))
#define spdQueuePeek(q, p)      _spdQueuePeek((q), (p))
#define spdQueuePoke(q, p, v)   _spdQueuePoke((q), (p), (v))
#define spdQueueGetFront(q)     _spdQueueGetFront((q))
#define spdQueueGetBack(q)      _spdQueueGetBack((q))
#define spdQueueSearchIdx(q, v) _spdQueueSearchIdx((q), (v))

#endif /* not SPD_QUEUE_UNION_TYPE_ELEM */

#endif /* not _SPD_QUEUE_H */
