/* 
 * Program:             $RCSfile: spdSet.h,v $  $Revision: 30.4 $
 * 
 * Purpose:             
 * 
 * Author:              M.Hakamata  1997/01/30
 *                      A.Watanabe  1997/01/30
 *                      S.Yamamoto  1997/02/09
 * 
 * (C) Copyright:       M.Hakamata, A.Watanabe and S.Yamamoto  1997 - 2003
 *                      This file is a product of the project Sapid.
 */

#ifndef _SPDSET_H
#define _SPDSET_H

#if defined(__STDC__) || defined(__cplusplus)
typedef void    *Contents;
#else
typedef int     *Contents;
#endif

/* 
 * If you want to use hash-key as real element and type of element is
 * "int", specify this macro when you call spdCreateSet1().
 */
#define DEFAULT_INT_SET     (0)
#define _ELEMPTR_BLKSIZ     (1024)

/*
 * case i) DEFULAT_INT_SET was specified.
 *
 *            +------+                 +------+
 *            |      |                 |      |
 *          + V ---- | ---+          + V ---- | ---+
 * listp--> |key, contents|--next--> |key, contents|--next--> NULL
 *          +-------------+          +-------------+
 *            ElemPtr                   ElemPtr
 *
 * case ii) sizeof(REAL ELEMENT) was specified
 *
 *            ElemPtr                   ElemPtr
 *          +-------------+          +-------------+
 * listp--> |key, contents|--next--> |key, contents|--next--> NULL
 *          +----- | -----+          +----- | -----+
 *                 V                        V
 *          +------------+           +------------+
 *          |real element|           |real element|
 *          +------------+           +------------+
 */

typedef struct _ElemPtr {
    /* Private members. */
    int             key;        /* Key (and real element if you specified 
                                   DEFAULT_INT_SET). */
    struct _ElemPtr *next;      /* Pointer to next ELemPtr. */
    Contents        contents;   /* Pointer to real element. */
    signed int      mark;       /* For freeList. */
} ElemPtr;

/*
 * Structure of ElemPtrBlk:
 *      O: occupied 
 *      F: freed
 *      U: unused
 * 
 * previous block (fully occupied)
 *    |                                
 *   prev   freeList     +--next--+     +--> NULL
 *    |             \   /          \   /
 *  +-|--+----+----+-V-|+----+----+-V-|+----+----+------+----+
 *  |head| O  | O  | F  | O  | O  | F  | O  | U  | ...  | U  |
 *  +----+----+----+----+----+----+----+----+----+------+----+
 *       +----------------------------------+----------------+
 *         0 ... (#usedNum - 1)               UNUSED ELEMPTR
 *       +---------------------------------------------------+
 *         0 ... (#_ELEMPTR_BLKSIZ - 1)
 */

typedef struct _ElemPtrBlk {
    /* Private members. */
    struct _ElemPtrBlk *prev;
    unsigned int       usedNum;
    ElemPtr            elemP[_ELEMPTR_BLKSIZ];
} ElemPtrBlk;

/*
 *   SpdSet1
 *   |
 *   V    _SpdSet1
 * +------------+              Array of buckets (hash table)
 * | bktNum     |              +---+          
 * | elemNum    |--bktAry--> #0|   |--listp--> ...
 * | rebuildNum |              +---+         
 * | elemSiz    |            #1|   |--listp--> NULL (if bucket is empty)
 * +-|----------+              +---+
 *   |                         | : |
 *   |                         | : |
 * elemPBlkList                +---+
 *   |             #bktNum - 1 |   |--listp--> ...
 *   |                         +---+
 *   | 
 *   |   +---prev-----+     +--prev--.....--+     +--prev--> NULL
 *   |  /              \   /                 \   /
 *  +V-|+---+---+---+  +V-|+---+---+---+     +V-|+---+---+---+
 *  |   |   |   |   |  |   |   |   |   | ... |   |   |   |   |
 *  +---+---+---+---+  +---+---+---+---+     +---+---+---+---+
 *  ElemPtrBlk         ElemPtrBlk            ElemPtrBlk 
 *  (not fully used)   (alreday fully used)  (alreday fully used)
 */

typedef struct _Bucket {
    /* Private members. */
    unsigned int    elemNum;    /* Number of ElemPtr list's elements. */
    ElemPtr        *listp;      /* Pointer to the first element of
                                   ElemPtr list. */
} Bucket;

typedef Bucket *BktArray;

typedef struct {
    /* Public members. */
    unsigned long   elemNum;        /* Number of elements. */
    size_t          elemSiz;        /* Size of element. */

    /* Private members */
    unsigned int    bktNum;         /* Number of buckets. */
    unsigned long   rebuildNum;     /* Rebuild hash table if elemNum gets to be
                                       larger more than this value. */
    BktArray        bktAry;         /* Pointer to array of buckets. */
    ElemPtr         *freeList;      /* Pointer to list of freed ELEMPTR. */
    ElemPtrBlk      *elemPBlkList;  /* Pointer to list of allocated ELEMPTRBLK. */
    ElemPtrBlk      *memory;        /* Used for spdGetAElem*(). */
    unsigned int    memIndex;       /* Used for spdGetAElem*(). */
} _SpdSet1;

typedef _SpdSet1 *SpdSet1;

/* Basic operation. */
SpdBoolean spdSetContains1(SpdSet1 set, int key);
Contents spdAddElem1(SpdSet1 set, int key, Contents contents);
int spdDelElem1(SpdSet1 set, int key);
SpdSet1 spdCreateSet1(size_t elemSiz);
void spdFreeSet1(SpdSet1 set);
Contents spdSearchElem1(SpdSet1 set, int key);
Contents spdGetArrayOfElem1(SpdSet1 set);
Contents spdGetAElem1(SpdSet1 set, int *key);

/* Set Operation. */
SpdSet1 spdDupSet1(SpdSet1 set);
SpdSet1 spdUnionSet1(SpdSet1 s1, SpdSet1 s2);
SpdSet1 spdIntersectionSet1(SpdSet1 s1, SpdSet1 s2);
SpdSet1 spdDifferenceSet1(SpdSet1 s1, SpdSet1 s2);

/* Misc. */
void spdClearSetEnumeration1(SpdSet1 set);

#ifdef SPDSET_DEBUG
void spdPrintSet1(SpdSet1 set);
void spdStatSet1(SpdSet1 set); 
#endif

#define SAPID_EMPTY_SET1       spdCreateSet1(DEFAULT_INT_SET)

#endif /* _SPDSET_H */
